@@ -363,6 +363,7 @@ typedef enum {
     CHIP_FAMILY_BARTS,
     CHIP_FAMILY_TURKS,
     CHIP_FAMILY_CAICOS,
+    CHIP_FAMILY_CAYMAN,
     CHIP_FAMILY_LAST
 } RADEONChipFamily;
 
@@ -494,7 +495,7 @@ typedef struct _atomBiosHandle *atomBiosHandlePtr;
 
 struct radeon_exa_pixmap_priv {
     struct radeon_bo *bo;
-    int flags;
+    uint32_t tiling_flags;
     Bool bo_mapped;
 };
 
@@ -683,6 +684,7 @@ struct r600_accel_object {
     int bpp;
     uint32_t domain;
     struct radeon_bo *bo;
+    uint32_t tiling_flags;
 };
 
 struct radeon_vbo_object {
@@ -715,6 +717,7 @@ struct radeon_accel_state {
     Bool              XInited3D; /* X itself has the 3D context */
     int               num_gb_pipes;
     Bool              has_tcl;
+    Bool              allowHWDFS;
 
 #ifdef USE_EXA
     /* EXA */
@@ -1066,6 +1069,7 @@ typedef struct {
     uint64_t gart_size;
     drmmode_rec drmmode;
     /* r6xx+ tile config */
+    Bool have_tiling_info;
     uint32_t tile_config;
     int group_bytes;
     int num_channels;
@@ -1287,7 +1291,7 @@ extern void RADEONPMFini(ScrnInfoPtr pScrn);
 #ifdef USE_EXA
 /* radeon_exa.c */
 extern Bool RADEONSetupMemEXA(ScreenPtr pScreen);
-extern Bool radeon_transform_is_affine(PictTransformPtr t);
+extern Bool radeon_transform_is_affine_or_scaled(PictTransformPtr t);
 
 /* radeon_exa_funcs.c */
 extern void RADEONCopyCP(PixmapPtr pDst, int srcX, int srcY, int dstX,
@@ -1310,7 +1314,6 @@ extern Bool R600DrawInit(ScreenPtr pScreen);
 extern Bool R600LoadShaders(ScrnInfoPtr pScrn);
 #ifdef XF86DRM_MODE
 extern Bool EVERGREENDrawInit(ScreenPtr pScreen);
-extern Bool EVERGREENLoadShaders(ScrnInfoPtr pScrn);
 #endif
 #endif
 
@@ -1381,6 +1384,7 @@ void radeon_kms_update_vram_limit(ScrnInfoPtr pScrn, int new_fb_size);
 #endif
 struct radeon_bo *radeon_get_pixmap_bo(PixmapPtr pPix);
 void radeon_set_pixmap_bo(PixmapPtr pPix, struct radeon_bo *bo);
+uint32_t radeon_get_pixmap_tiling(PixmapPtr pPix);
 
 #ifdef XF86DRI
 #  ifdef USE_XAA
