@@ -247,8 +247,12 @@ RADEONPutImageTextured(ScrnInfoPtr pScrn,
     int top, nlines, size;
     BoxRec dstBox;
     int dst_width = width, dst_height = height;
-    int hw_align;
-
+    int aligned_height;
+#ifdef XF86DRM_MODE
+    int h_align = drmmode_get_height_align(pScrn, 0);
+#else
+    int h_align = 1;
+#endif
     /* make the compiler happy */
     s2offset = s3offset = srcPitch2 = 0;
 
@@ -284,10 +288,22 @@ RADEONPutImageTextured(ScrnInfoPtr pScrn,
 	    pPriv->bicubic_enabled = FALSE;
     }
 
-    if (info->ChipFamily >= CHIP_FAMILY_R600)
-	hw_align = 256;
-    else
-	hw_align = 64;
+#ifdef XF86DRM_MODE
+    if (info->cs) {
+	if (info->ChipFamily >= CHIP_FAMILY_R600)
+	    pPriv->hw_align = drmmode_get_base_align(pScrn, 2, 0);
+	else
+	    pPriv->hw_align = 64;
+    } else
+#endif
+    {
+	if (info->ChipFamily >= CHIP_FAMILY_R600)
+	    pPriv->hw_align = 256;
+	else
+	    pPriv->hw_align = 64;
+    }
+
+    aligned_height = RADEON_ALIGN(dst_height, h_align);
 
     switch(id) {
     case FOURCC_YV12:
@@ -295,24 +311,24 @@ RADEONPutImageTextured(ScrnInfoPtr pScrn,
 	srcPitch = RADEON_ALIGN(width, 4);
 	srcPitch2 = RADEON_ALIGN(width >> 1, 4);
         if (pPriv->bicubic_state != BICUBIC_OFF) {
-	    dstPitch = RADEON_ALIGN(dst_width << 1, hw_align);
+	    dstPitch = RADEON_ALIGN(dst_width << 1, pPriv->hw_align);
 	    dstPitch2 = 0;
 	} else {
-	    dstPitch = RADEON_ALIGN(dst_width, hw_align);
-	    dstPitch2 = RADEON_ALIGN(dstPitch >> 1, hw_align);
+	    dstPitch = RADEON_ALIGN(dst_width, pPriv->hw_align);
+	    dstPitch2 = RADEON_ALIGN(dstPitch >> 1, pPriv->hw_align);
 	}
 	break;
     case FOURCC_UYVY:
     case FOURCC_YUY2:
     default:
-	dstPitch = RADEON_ALIGN(dst_width << 1, hw_align);
+	dstPitch = RADEON_ALIGN(dst_width << 1, pPriv->hw_align);
 	srcPitch = (width << 1);
 	srcPitch2 = 0;
 	break;
     }
 
-    size = dstPitch * dst_height + 2 * dstPitch2 * ((dst_height + 1) >> 1);
-    size = RADEON_ALIGN(size, hw_align);
+    size = dstPitch * aligned_height + 2 * dstPitch2 * RADEON_ALIGN(((aligned_height + 1) >> 1), h_align);
+    size = RADEON_ALIGN(size, pPriv->hw_align);
 
     if (size != pPriv->size) {
 	RADEONFreeVideoMemory(pScrn, pPriv);
@@ -321,7 +337,7 @@ RADEONPutImageTextured(ScrnInfoPtr pScrn,
     if (pPriv->video_memory == NULL) {
 	pPriv->video_offset = radeon_legacy_allocate_memory(pScrn,
 							    &pPriv->video_memory,
-							    size, hw_align,
+							    size, pPriv->hw_align,
 							    RADEON_GEM_DOMAIN_GTT);
 	if (pPriv->video_offset == 0)
 	    return BadAlloc;
@@ -329,7 +345,7 @@ RADEONPutImageTextured(ScrnInfoPtr pScrn,
 	if (info->cs) {
 	    pPriv->src_bo[0] = pPriv->video_memory;
 	    radeon_legacy_allocate_memory(pScrn, (void*)&pPriv->src_bo[1], size,
-					  hw_align,
+					  pPriv->hw_align,
 					  RADEON_GEM_DOMAIN_GTT);
 	}
     }
@@ -389,10 +405,10 @@ RADEONPutImageTextured(ScrnInfoPtr pScrn,
     }
     pPriv->src_pitch = dstPitch;
 
-    pPriv->planeu_offset = dstPitch * dst_height;
-    pPriv->planeu_offset = RADEON_ALIGN(pPriv->planeu_offset, hw_align);
-    pPriv->planev_offset = pPriv->planeu_offset + dstPitch2 * ((dst_height + 1) >> 1);
-    pPriv->planev_offset = RADEON_ALIGN(pPriv->planev_offset, hw_align);
+    pPriv->planeu_offset = dstPitch * aligned_height;
+    pPriv->planeu_offset = RADEON_ALIGN(pPriv->planeu_offset, pPriv->hw_align);
+    pPriv->planev_offset = pPriv->planeu_offset + dstPitch2 * RADEON_ALIGN(((aligned_height + 1) >> 1), h_align);
+    pPriv->planev_offset = RADEON_ALIGN(pPriv->planev_offset, pPriv->hw_align);
 
     pPriv->size = size;
     pPriv->pDraw = pDraw;
